-- Author: Max David aka Vènomstrikè
-- LibArena is a Library to get informations in/about a Arena match
-- Designed for VFrame, for usage look up in the VFrame Code or the API Documentation!

---------Setting the lib-----------

local version = 1.7

lib = LibStub:NewLibrary("LibArena-1.0", version)

lib.callbacks = LibStub("CallbackHandler-1.0"):New(lib)

if not lib then print("Error while creating Library!"); return end
if not lib.callbacks then print("Can't find CallbackHandler-1.0!"); return end

local classtbl = {
	[62] = "Mage: Arcane",
	[63] = "Mage: Fire",
	[64] = "Mage: Frost",
	[65] = "Paladin: Holy",
	[66] = "Paladin: Protection",
	[70] = "Paladin: Retribution",
	[71] = "Warrior: Arms",
	[72] = "Warrior: Fury",
	[73] = "Warrior: Protection",
	[102] = "Druid: Balance",
	[103] = "Druid: Feral",
	[104] = "Druid: Guardian",
	[105] = "Druid: Restoration",
	[250] = "Death Knight: Blood",
	[251] = "Death Knight: Frost",
	[252] = "Death Knight: Unholy",
	[253] = "Hunter: Beast Mastery",
	[254] = "Hunter: Marksmanship",
	[255] = "Hunter: Survival",
	[256] = "Priest: Discipline",
	[257] = "Priest: Holy",
	[258] = "Priest: Shadow",
	[259] = "Rogue: Assassination",
	[260] = "Rogue: Combat",
	[261] = "Rogue: Subtlety",
	[262] = "Shaman: Elemental",
	[263] = "Shaman: Enhancement",
	[264] = "Shaman: Restoration",
	[265] = "Warlock: Affliction",
	[266] = "Warlock: Demonology",
	[267] = "Warlock: Destruction",
	[268] = "Monk: Brewmaster",
	[269] = "Monk: Windwalker",
	[270] = "Monk: Mistweaver",
}

local classcolor = {
	[62] = "0.41-0.80-0.94",
	[63] = "0.41-0.80-0.94",
	[64] = "0.41-0.80-0.94",
	[65] = "0.96-0.55-0.73",
	[66] = "0.96-0.55-0.73",
	[70] = "0.96-0.55-0.73",
	[71] = "0.78-0.61-0.43",
	[72] = "0.78-0.61-0.43",
	[73] = "0.78-0.61-0.43",
	[102] = "1.00-0.49-0.04",
	[103] = "1.00-0.49-0.04",
	[104] = "1.00-0.49-0.04",
	[105] = "1.00-0.49-0.04",
	[250] = "0.77-0.12-0.23",
	[251] = "0.77-0.12-0.23",
	[252] = "0.77-0.12-0.23",
	[253] = "0.67-0.83-0.45",
	[254] = "0.67-0.83-0.45",
	[255] = "0.67-0.83-0.45",
	[256] = "1.00-1.00-1.00",
	[257] = "1.00-1.00-1.00",
	[258] = "1.00-1.00-1.00",
	[259] = "1.00-0.96-0.41",
	[260] = "1.00-0.96-0.41",
	[261] = "1.00-0.96-0.41",
	[262] = "0.00-0.44-0.87",
	[263] = "0.00-0.44-0.87",
	[264] = "0.00-0.44-0.87",
	[265] = "0.58-0.51-0.79",
	[266] = "0.58-0.51-0.79",
	[267] = "0.58-0.51-0.79",
	[268] = "0.00-1.00-0.59",
	[269] = "0.00-1.00-0.59",
	[270] = "0.00-1.00-0.59",
}

local UBScount = 0
frameUBS = CreateFrame("Frame", nil, nil)
framePEW = CreateFrame("Frame", nil, nil)


----------Methode Part-------------

function lib:getMapIndex(mapName, eng, loc)

	locale = loc or GetLocale()

	deDE = { -- German
		[1] = "Ruinen von Lordaeron",
		[2] = "Kanalisation von Dalaran",
		[3] = "Arena von Nagrand",
		[4] = "Der Tigergipfel",
		[5] = "Arena des Schergrats",
		[6] = "Arena der Tol'vir"
	}

	enUS = { -- American English
		[1] = "Ruins of Lordaeron",
		[2] = "Dalaran Sewers",
		[3] = "Nagrand Arena",
		[4] = "The Tiger's Peak",
		[5] = "Blade's Edge Arena",
		[6] = "Tol'Viron Arena"
	}

	esES = { -- Spanish (European)
		[1] = "Ruinas de Lordaeron",
		[2] = "Cloacas de Dalaran",
		[3] = "Arena de Nagrand",
		[4] = "La Cima del Tigre",
		[5] = "Arena Filospada",
		[6] = "Arena Tol'viron"
	}

	frFR = { -- French
		[1] = "Ruines de Lordaeron",
		[2] = "Égouts de Dalaran",
		[3] = "Arène de Nagrand",
		[4] = "Le croc du Tigre",
		[5] = "Arène des Tranchantes",
		[6] = "Arène Tol'viron"
	}

	ruRU = { -- Russian
		[1] = "Руины Лордерона",
		[2] = "Стоки Даларана",
		[3] = "Арена Награнда",
		[4] = "Пик Тигра",
		[5] = "Арена Острогорья",
		[6] = "Арена Тол'вир"
	}

	koKR = { -- Korean
		[1] = "로데론의 폐허",
		[2] = "달라란 하수도",
		[3] = "나그란드 투기장",
		[4] = "범의 봉우리",
		[5] = "칼날 산맥 투기장",
		[6] = "톨비론 투기장"
	}

	zhTW = { -- Chinese (traditional; Taiwan)
		[1] = "羅德隆廢墟",
		[2] = "達拉然下水道",
		[3] = "納葛蘭競技場",
		[4] = "猛虎峰",
		[5] = "劍刃競技場",
		[6] = "托維恩競技場"
	}


	if locale == "deDE" then localetbl = deDE end
	if locale == "enGB" then localetbl = enUS end
	if locale == "enUS" then localetbl = enUS end
	if locale == "esES" then localetbl = esES end
	if locale == "esMX" then localetbl = esES end
	if locale == "frFR" then localetbl = frFR end
	if locale == "ruRU" then localetbl = ruRU end
	if locale == "koKR" then localetbl = koKR end
	if locale == "zhCN" then localetbl = zhTW end
	if locale == "zhTW" then localetbl = zhTW end


	if eng then
		for i,n in pairs(localetbl) do
			if n == mapName then return enUS[i] end
		end
	else
		for i,n in pairs(localetbl) do
			if n == mapName then return i end
		end
	end

	return nil

end

function lib:getSpecColor(spec)
	
	raw = classcolor[spec]
	rgb = split(raw, "-", true)

	return rgb[1], rgb[2], rgb[3]

end

function lib:getSpecIdFromName(name, classToken) -- getting the specID from a specName in any language [Specnames like Frost are exsisting multiple times so we need the class too] 

	if not name then return end

	for i,n in pairs(classtbl) do	
		id, specName, description, icon, background, role, specClass = GetSpecializationInfoByID(i)
		if specName == name and specClass == classToken then return i end
	end

end

function lib:getRankedInfo(size)
	
	rating, seasonBest, weeklyBest, plays, wins, weeklyPlayed, weeklyWon, cap = GetPersonalRatedInfo(size)

	looses = plays - wins
	zw = wins/plays
	ratio = zw*100

	return wins, looses, math.floor(ratio), rating

end

function lib:IsArenaEnemy(unit) -- returns only true when it is an arena unit and not unknown

	if unit == "arena1" then 
		return true 
	elseif unit == "arena2" then 
		return true 
	elseif unit == "arena3" then 
		return true 
	elseif unit == "arena4" then 
		return true 
	elseif unit == "arena5" then 
		return true 
	else
		return false
	end

end

function lib.callbacks:OnUsed(target, eventname)

	framePEW:RegisterEvent("PLAYER_ENTERING_WORLD")

	if eventname == "MATCH_INFO_UPDATE" then framePEW:SetScript("OnEvent", function() PLAYER_ENTERING_WORLD() end ) end

end

-------------------------------
----- GetPlayerInfos Part -----
-------------------------------

local IsInArena = false
function PLAYER_ENTERING_WORLD() -- fires when arena is entered or leaved

	instanceType = select(2, IsInInstance())

	-- check if we are entering or leaving an arena
	if instanceType == "arena" then	
			--------------------

			UBScount = 0

			-- Arena Entered
			IsInArena = true

			frameUBS:RegisterEvent("UPDATE_BATTLEFIELD_STATUS")
			frameUBS:SetScript("OnEvent", function() UPDATE_BATTLEFIELD_STATUS() end)
			

			--------------------
	else
		if IsInArena == true then
			--------------------

			-- Arena Leaved
			UBScount = 0

			frameUBS:SetScript("OnEvent", nil)

			IsInArena = false

			--------------------
		end
	end 

end

function UPDATE_BATTLEFIELD_STATUS() -- Fires when the arena begins and ends 
	UBScount = UBScount + 1

	if UBScount == 2 then
		handlePlayedMatch()
	end

end

function handlePlayedMatch()
	total = 0
	used = false
		 
		local function onUpdate(self, elapsed)
		    total = total + elapsed
		    if total >= 3 then
		    	if not used then
		    		GetMatchInfo()
		    		used = true
		    	end
		    end
		end
		 
		local f = CreateFrame("frame")
		f:SetScript("OnUpdate", onUpdate)

end

function GetMatchInfo()
	UBScount = -5 -- when arena ends and the Type of the Match was Skirmish, -5 will let the player que up for the next Match without an Error.

	players = {}
	match = {}

	-----Getting infos for all players in the match------
	num = 1
	repeat

		infos = {}
		infos.name, infos.kills, infos.honorableKills, infos.deaths, infos.honorGained, infos.faction, infos.race, infos.class, infos.classToken, infos.dmg, infos.heal, infos.rating, infos.ratingChange, infos.preMatchMMR, infos.mmrChange, infos.specName = GetBattlefieldScore(num)
		infos.spec = lib:getSpecIdFromName(infos.specName, infos.classToken)
		players[num] = infos

		if not infos.name and num == 1 then lib.callbacks:Fire("MATCH_INFO_UPDATE", false, nil); return end
		num = num + 1
		

	until not select(1, GetBattlefieldScore(num))
	-----------------------------------

	-----Getting infos about the match--------
	ownFac = GetBattlefieldArenaFaction()
	if ownFac == GetBattlefieldWinner() then win = true else win = false end
	status, mapName, instanceID, bracketMin, bracketMax, teamSize, registeredMatch = GetBattlefieldStatus(1)
	if not registeredMatch then registeredMatch = 0 end

	match.win = win
	match.time = (GetBattlefieldInstanceRunTime()/1000)
	match.map = mapName
	match.ownFac = ownFac
	newPlayers, match.size = getArenaPlayers(players, ownFac)
	riNum = tonumber(split(match.size, "v", false))
	if riNum == 1 or riNum == 2 then riNum = 1 elseif riNum == 3 then riNum = 2 elseif riNum == 4 or riNum == 5 then riNum = 3 end
	match.rating = tonumber(select(4, lib:getRankedInfo(riNum)))

	if teamSize == "ARENASKIRMISH" then match.ranked = false else match.ranked = true end

	lib.callbacks:Fire("MATCH_INFO_UPDATE", match, newPlayers)

end

function getArenaPlayers(players, ownFac) 

	enemys = {}
	team = {}

	mi = 0
	ei = 0
	for i,n in pairs(players) do
		infos = n
		if infos.faction == ownFac then
			mi = mi + 1
			member = {}
				member.dmg = infos.dmg
				member.heal = infos.heal
				member.name = infos.name
				member.spec = infos.spec
				member.race = infos.race
				if infos.dmg == 0 and infos.heal == 0 and infos.honorableKills == 0 then member.state = "cleared" else member.state = "normal" end
			team[mi] = member
		else
			ei = ei + 1
			enemy = {}
				enemy.dmg = infos.dmg
				enemy.heal = infos.heal
				enemy.name = infos.name
				enemy.spec = infos.spec
				enemy.race = infos.race
				if infos.dmg == 0 and infos.heal == 0 and infos.honorableKills == 0 then enemy.state = "cleared" else enemy.state = "normal" end
			enemys[ei] = enemy
		end
	end

	oei = ei
	omi = mi

	if mi <= 5 then
		repeat
			mi = mi + 1
			member = {}
				member.dmg = 0
				member.heal = 0
				member.name = "ni"
				member.spec = 0
				member.state = "ni"
			team[mi] = member
		until mi == 5 
	end

	if ei <= 5 then
		repeat
			ei = ei + 1
			enemy = {}
				enemy.dmg = 0
				enemy.heal = 0
				enemy.name = "ni"
				enemy.spec = 0
				enemy.state = "ni"
			enemys[ei] = enemy
		until ei == 5 
	end

	newPlayers = {}
	newPlayers.enemys = enemys
	newPlayers.team = team

	size = omi .. "v" .. oei

	return newPlayers, size

end

-------------------------------
-------------------------------

function split(inputstr, sep, tbl) -- splits a string


        local t={} ; i=1
        for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
                t[i] = str
                i = i + 1
        end

        if tbl then return t else return t[1] end

end

function round(n) -- rounds a value 

    return n % 1 >= 0.5 and math.ceil(n) or math.floor(n)

end

------------------------------------